@@ -1,52 +1,52 @@
-import gzip
-import io
 import pysam
-import requests
-from intervaltree import IntervalTree
-
+from igv_reports.featureTree import FeatureTree
+from igv_reports.chralias import get_alias
+from igv_reports.stream import getstream
 
 class Feature:
 
-    def __init__(self, chr, start, end, text, name=''):
+    def __init__(self, chr, start, end, text, name='', chr2=None, start2=None, end2=None):
         self.chr = chr
         self.start = start
         self.end = end
+        self.chr2 = chr2
+        self.start2 = start2
+        self.end2 = end2
         self.text = text
         self.name = name
 
+'''
+Return an appropriate reader for the given path
+'''
+def get_featurereader(path):
 
-class FeatureReader:
+    if path.endswith(".gz"):
+        # Might be a tabix file
+        try:
+            return TabixFeatureReader(pysam.TabixFile(path))
+        except:
+            return FeatureReader(path)
 
-    def __init__(self, path):
-
-        tabix = None
-        if path.endswith(".gz"):
-            # Might be a tabix file
-            try:
-                tabix = pysam.TabixFile(path)
-            except:
-                tabix = None
-
-        if tabix:
-            self.reader = _Tabix(tabix)
-        else:
-            self.reader = _NonIndexed(path)
-
-    def slice(self, region=None):
-        return self.reader.slice(region)
+    else:
+        return FeatureReader(path)
 
 
-class _Tabix:
+'''
+Tabix indexed file reader
+'''
+
+class TabixFeatureReader:
 
     def __init__(self, tabix):
         self.file = tabix
+        self.aliastable = {}
 
-    def slice(self, region=None):
+    def slice(self, region=None, region2=None):
 
         tb = self.file
-        if region:
-            range_string = region['chr'] + ":" + str(region['start']) + "-" + str(region['end'])
-            it = tb.fetch(range_string)
+        if region is not None:
+            chr = self.get_chrname(region["chr"])
+            it = self._fetch(chr, region["start"], region["end"])
         else:
             it = tb.fetch()
 
@@ -54,18 +54,57 @@ class _Tabix:
         for row in it:
             data += row + '\n'
 
+        if region2 is not None:
+            chr = self.get_chrname(region2["chr"])
+            it = self._fetch(chr, region2["start"], region2["end"])
+            for row in it:
+                data += row + '\n'
+
         return data
 
+    def _fetch(self, chr, start, end):
 
-## Implement a pysam/tabix style inteface for non-indexed files
-class _NonIndexed:
+        if chr is None:
+            return []
+
+        tb = self.file
+        try:
+            range_string = f'{chr}:{start}-{end}'
+            return tb.fetch(range_string)
+
+        except ValueError:
+            # Possible chr alias error (e.g. 1 vs chr1)
+            try:
+                alias = get_alias(chr)
+                range_string = f'{alias}:{start}-{end}'
+                value = tb.fetch(range_string)
+                self.aliastable[chr] = alias
+                return value
+            except ValueError:
+                self.aliastable[chr] = None
+                print(f'WARNING: No data with sequence name {chr} found in file {self.file.filename}')
+                return []
+
+    def get_chrname(self, c):
+        if c in self.aliastable:
+            return self.aliastable[c]
+        else:
+            return c
+
+
+'''
+Non-indexed file reader.  Emulates a tabix style query interface
+'''
+
+class FeatureReader:
 
     def __init__(self, file):
 
         self.file = file
         self.tree = None
+        self.aliastable = {}
 
-    def slice(self, region=None):
+    def slice(self, region=None, region2=None):
 
         if region == None:
             f = None
@@ -77,95 +116,106 @@ class _NonIndexed:
                     f.close()
 
         else:
+
+            # If first time through read the entire file and create a queyable feature tree.
             if not self.tree:
                 features = parse(self.file)
+                for f in features:
+                    if f.chr not in self.aliastable:
+                        self.aliastable[get_alias(f.chr)] = f.chr
                 self.tree = FeatureTree(features)
 
-            reference = region["chr"]
+            features = []
+
+            reference = self.get_chrname(region["chr"])
             start = region["start"]
             end = region["end"]
             feature_intervals = self.tree.query(reference, start, end)
-
-            features = []
             if feature_intervals:
-                features = []
                 for i in feature_intervals:
                     features.append(i.data)
 
-                def sortFunc(f):
-                    return f.start
-
-                features = sorted(features, key=sortFunc)
+            if region2 is not None:
+                reference = self.get_chrname(region2["chr"])
+                start = region2["start"]
+                end = region2["end"]
+                feature_intervals = self.tree.query(reference, start, end)
+                if feature_intervals:
+                    for i in feature_intervals:
+                        features.append(i.data)
 
         content = ''
         for f in features:
             content += f.text
         return content
 
+    def get_chrname(self, c):
+        if c in self.aliastable:
+            return self.aliastable[c]
+        else:
+            return c
+
+
+'''
+A "mock reader" initialized with a list of bed features
+'''
 
-# This class is initialized with a list of "bed like" features, mocks an indexed file reader
 class MockReader:
 
     def __init__(self, features):
-
+        self.features = features
         self.tree = FeatureTree(features)
-
-    def slice(self, region=None):
-
-        reference = region["chr"]
-        start = region["start"]
-        end = region["end"]
-        feature_intervals = self.tree.query(reference, start, end)
-
-        features = []
-        if feature_intervals:
-            features = []
-            for i in feature_intervals:
-                features.append(i.data)
-
-            def sortFunc(f):
-                return f.start
-
-            features = sorted(features, key=sortFunc)
-
-        content = ''
+        self.aliastable = {}
         for f in features:
-            content += f"{f.chr}\t{f.start}\t{f.end}\n"
-        return content
+            if f.chr not in self.aliastable:
+                self.aliastable[get_alias(f.chr)] = f.chr
 
+    def slice(self, region=None, region2=None):
 
-class FeatureTree:
+        if region == None:
+            return self.features
 
-    def __init__(self, featureList):
-
-        self.featureMap = dict();
-
-        for f in featureList:
-            chr = f.chr
-            tree = self.featureMap.get(chr)
-            if tree == None:
-                tree = IntervalTree()
-                self.featureMap[chr] = tree
-            tree[f.start:f.end] = f
-
-    def query(self, chr, start, end):
-        tree = self.featureMap.get(chr)
-        if tree == None:
-            return set()
         else:
-            return tree[start:end]
 
+            features = []
 
-def get_data(filename, region=None):
-    reader = FeatureReader(filename)
-    return reader.slice(region)
+            reference = self.get_chrname(region["chr"])
+            start = region["start"]
+            end = region["end"]
+            feature_intervals = self.tree.query(reference, start, end)
 
+            if feature_intervals:
+                for i in feature_intervals:
+                    features.append(i.data)
+
+            if region2 is not None:
+                reference = self.get_chrname(region2["chr"])
+                start = region2["start"]
+                end = region2["end"]
+                feature_intervals = self.tree.query(reference, start, end)
+
+                if feature_intervals:
+                    for i in feature_intervals:
+                        features.append(i.data)
+
+            content = ''
+            for f in features:
+                chr = f.chr
+                content += f"{chr}\t{f.start}\t{f.end}\n"
+            return content
+
+    def get_chrname(self, c):
+        if c in self.aliastable:
+            return self.aliastable[c]
+        else:
+            return c
 
 def parse(path, format=None):
     '''
     Parse a feature file and return an array of feature objects.  Supported formats are bed, gff, and gtf.
     :param path: Path to feature file, which can be local or url
     :param format: File format, bed | gtf | gff
+    :param split_bool: Boolean specifying whether view is multi locus or not
     :return: List of feature objects {chr, start, end, text, name}
     '''
 
@@ -176,10 +226,14 @@ def parse(path, format=None):
             format = infer_format(path)
         if format == 'bed':
             return parse_bed(f)
-        elif format == 'gff' or format == 'gtf':
+        elif format == 'gff'  or format == 'gff3' or format == 'gtf':
             return parse_gff(f)
         elif format == 'tab':
             return parse_tab(f)
+        elif format == 'bedpe':
+            return parse_bedpe(f)
+        elif format == 'refgene':
+            return parse_refgene(f)
         else:
             raise Exception("Unknown file format: " + path)
     finally:
@@ -192,7 +246,7 @@ def parse_bed(f):
     for line in f:
         if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
             tokens = line.rstrip('\n').rstrip('\r').split('\t')
-            if len(tokens) > 2:
+            if len(tokens) >= 3:
                 chr = tokens[0]
                 start = int(tokens[1])
                 end = int(tokens[2])
@@ -201,6 +255,37 @@ def parse_bed(f):
     return features
 
 
+def parse_refgene(f):
+    features = []
+    for line in f:
+        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
+            tokens = line.rstrip('\n').rstrip('\r').split('\t')
+            if len(tokens) >= 3:
+                chr = tokens[2]
+                start = int(tokens[4])
+                end = int(tokens[5])
+                name = tokens[12] if len(tokens) > 12 else ''
+                features.append(Feature(chr, start, end, line, name))
+    return features
+
+
+def parse_bedpe(f):
+    features = []
+    for line in f:
+        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
+            tokens = line.rstrip('\n').rstrip('\r').split('\t')
+            if len(tokens) >= 6:
+                chr = tokens[0]
+                start = int(tokens[1])
+                end = int(tokens[2])
+                chr2 = tokens[3]
+                start2 = int(tokens[4])
+                end2 = int(tokens[5])
+                name = tokens[6] if len(tokens) > 6 else ''
+                features.append(Feature(chr, start, end, line, name, chr2, start2, end2))
+    return features
+
+
 def parse_gff(f):
     features = []
     for line in f:
@@ -249,6 +334,8 @@ def infer_format(filename):
         return "vcf"
     elif filename.endswith(".bed"):
         return "bed"
+    elif filename.endswith(".bedpe"):
+        return "bedpe"
     elif filename.endswith(".gff") or filename.endswith(".gff3"):
         return "gff"
     elif filename.endswith(".gtf"):
@@ -266,27 +353,3 @@ def infer_format(filename):
             return filename[idx + 1:]
         else:
             return None
-
-
-def getstream(file):
-    # TODO -- gcs
-
-    if file.startswith('http://') or file.startswith('https://'):
-        response = requests.get(file)
-        status_code = response.status_code  # TODO Do something with this
-
-        if file.endswith('.gz'):
-            content = response.content
-            text = gzip.decompress(content).decode('utf-8')
-        else:
-            text = response.text
-        f = io.StringIO(text)
-        return text
-
-    elif file.endswith('.gz'):
-        f = gzip.open(file, mode='rt')
-
-    else:
-        f = open(file, encoding='UTF-8')
-
-    return f
