@@ -1,6 +1,8 @@
 from . import feature, bam, vcf
 
-def getreader(path, filetype=None, fasta=None):
+def getreader(config, filetype=None, fasta=None):
+
+    path = config["url"]
 
     if not filetype:
         filetype = feature.infer_format(path)
@@ -8,14 +10,17 @@ def getreader(path, filetype=None, fasta=None):
         filetype = filetype.lower()
 
     if filetype == "bam" :
+        return bam.BamReader(path)
+
+    if filetype == "cram":
         return bam.BamReader(path, fasta)
 
-    elif filetype == "cram":
-        return bam.BamReader(path, fasta)
 
-    elif filetype == "vcf":
+    elif filetype == "vcf" or filetype == "bcf":
         return vcf.VcfReader(path)
 
     else:
-        return feature.FeatureReader(path)
+        return feature.get_featurereader(path)
+
+
 
